define([
    'mage/utils/wrapper',
    'mage/storage',
    'Magento_Checkout/js/model/error-processor',
    'Magento_Checkout/js/model/full-screen-loader',
    'Magento_Customer/js/customer-data',
    'Magento_Checkout/js/model/payment/place-order-hooks',
    'underscore',
    'mage/translate'
], function (wrapper, storage, errorProcessor, fullScreenLoader, customerData, hooks, _, $t) {
    'use strict';

    return function (placeOrder) {
        return wrapper.wrap(placeOrder, function (originalPlaceOrder, serviceUrl, payload, messageContainer) {
            var headers = {};

            fullScreenLoader.startLoader();
            _.each(hooks.requestModifiers, function (modifier) {
                modifier(headers, payload);
            });

            return storage.post(
                serviceUrl, JSON.stringify(payload), true, 'application/json', headers
            ).fail(
                function (response) {
                    if (payload.paymentMethod !== undefined && payload.paymentMethod.method === 'stripe_payments') {
                        var box = document.getElementById('stripe-payments-card-errors');
                        if (box) {
                            box.innerText = response.responseJSON.message !== undefined ? response.responseJSON.message : $t('Invalid card information.');
                            box.classList.add('populated');
                        } else
                            alert(message);
                    }
                    errorProcessor.process(response, messageContainer);
                }
            ).done(
                function (response) {
                    var clearData = {
                        'selectedShippingAddress': null,
                        'shippingAddressFromData': null,
                        'newCustomerShippingAddress': null,
                        'selectedShippingRate': null,
                        'selectedPaymentMethod': null,
                        'selectedBillingAddress': null,
                        'billingAddressFromData': null,
                        'newCustomerBillingAddress': null
                    };

                    if (response.responseType !== 'error') {
                        customerData.set('checkout-data', clearData);
                    }
                }
            ).always(
                function () {
                    fullScreenLoader.stopLoader();
                    _.each(hooks.afterRequestListeners, function (listener) {
                        listener();
                    });
                }
            );
        });
    };
});
